#include "nvs_flash.h"
#include "esp_netif.h"
#include "zh_rf24.h"
#ifdef CONFIG_IDF_TARGET_ESP8266
#include "esp_system.h"
#else
#include "esp_random.h"
#endif

void zh_rf24_event_handler(void *arg, esp_event_base_t event_base, int32_t event_id, void *event_data);

typedef struct
{
    char char_value[10];
    int int_value;
    float float_value;
    bool bool_value;
} example_message_t;

void app_main(void)
{
    // esp_log_level_set("zh_rf24", ESP_LOG_NONE);
    nvs_flash_init();
    esp_netif_init();
    esp_event_loop_create_default();
    zh_rf24_init_config_t zh_rf24_init_config = ZH_RF24_INIT_CONFIG_DEFAULT();
    zh_rf24_init(&zh_rf24_init_config);
#ifdef CONFIG_IDF_TARGET_ESP8266
    esp_event_handler_register(ZH_RF24, ESP_EVENT_ANY_ID, &zh_rf24_event_handler, NULL);
#else
    esp_event_handler_instance_register(ZH_RF24, ESP_EVENT_ANY_ID, &zh_rf24_event_handler, NULL, NULL);
#endif
    example_message_t send_message = {0};
    strcpy(send_message.char_value, "CHAR");
    send_message.float_value = 1.234;
    send_message.bool_value = false;
    for (;;)
    {
        send_message.int_value = esp_random();
        // zh_rf24_send((uint8_t *)&send_message, sizeof(send_message), true);
        vTaskDelay(5000 / portTICK_PERIOD_MS);
    }
}

void zh_rf24_event_handler(void *arg, esp_event_base_t event_base, int32_t event_id, void *event_data)
{
    switch (event_id)
    {
    case ZH_RF24_ON_RECV_EVENT:;
        zh_rf24_event_on_recv_t *recv_data = event_data;
        printf("Message via pipe %d is received. Data lenght %d bytes.\n", recv_data->pipe, recv_data->data_len);
        example_message_t *recv_message = (example_message_t *)recv_data->data;
        printf("Char %s\n", recv_message->char_value);
        printf("Int %d\n", recv_message->int_value);
        printf("Float %f\n", recv_message->float_value);
        printf("Bool %d\n", recv_message->bool_value);
        free(recv_data->data); // Do not delete to avoid memory leaks!
        break;
    case ZH_RF24_ON_SEND_EVENT:;
        zh_rf24_event_on_send_t *send_data = event_data;
        if (send_data->status == ZH_RF24_SEND_SUCCESS)
        {
            printf("Message sent success.\n");
        }
        else
        {
            printf("Message sent fail.\n");
        }
    default:
        break;
    }
}